###################### Control Sinttico ####################
# Ejemplo: Abadie, Diamond, Hainmueller (2015)

### Clear R environment
rm(list = ls(all = TRUE))

### Install R library
#install.packages(c( 'scpi','ggplot2','dplyr'))

### Load package
library(scpi)
library(ggplot2)
library(dplyr)

#directorios.
#aqu cambiar a ruta en computadora personal
proyecto="//Bmdgiesan/DGIESAN/PROYECTOS/DASPERI/DatosInvestigacin/CursoDD/ejemplos/control_sintetico"
datos = paste0(proyecto,"/datos")
figuras = paste0(proyecto,"/figuras")
 
################################################################
# cargar datos
data <- scpi_germany

# explorar datos
head(data)
summary(data)
View(data)

#####################################################################################
#                             1. Estimacin
####################################################################################
# ver figura 2 en Abadie et al (2015)
####################################
### Set options for data preparation
id.var      <- "country"                              # ID variable
time.var    <- "year"                                 # Time variable
period.pre  <- seq(from = 1960, to = 1990, by = 1)    # Pre-treatment period
period.post <- (1991:2003)                            # Post-treatment period
unit.tr     <- "West Germany"                         # Treated unit (in terms of id.var)
unit.co     <- setdiff(unique(data$country), unit.tr) # Donors pool
outcome.var <- "gdp"                                  # Outcome variable


####################################
### Data preparation
df  <-   scdata(df = data, id.var = id.var, time.var = time.var, outcome.var = outcome.var,
                period.pre = period.pre, period.post = period.post, 
                unit.tr = unit.tr, unit.co = unit.co)

####################################
### SC - point estimation with simplex (ver otros tipos de restriccin en pag. 12 en paper de SCPI)
est.si  <- scest(data = df, w.constr = list(name="simplex"))
# Use print or summary methods to check results
print(est.si)
summary(est.si)

####################################
### SC - plot results
scplot(result = est.si, fig.path = figuras,
       fig.name = "germany_est", fig.format = "png", plot.range = (1960:2003),
       label.xy = list(x.lab = "Year", y.lab = "GDP per capita (thousand US dollars)"),
       event.label = list(lab = "Reunification", height = 10),
       plot.specs=list( img.width = 7,  img.height = 5, dpi=100))


#####################################################################################
#                  2. Prueba de adelanto de fecha (backdating)
####################################################################################
# ver Figura 4 en Abadie et al (2015)

####################################
### Set options for data preparation
id.var      <- "country"                              # ID variable
time.var    <- "year"                                 # Time variable
period.pre  <- seq(from = 1960, to = 1980, by = 1)    # Pre-treatment period
period.post <- (1981:2003)                            # Post-treatment period
unit.tr     <- "West Germany"                         # Treated unit (in terms of id.var)
unit.co     <- setdiff(unique(data$country), unit.tr) # Donors pool
outcome.var <- "gdp"                                  # Outcome variable

####################################
### Data preparation
df  <-   scdata(df = data, id.var = id.var, time.var = time.var, outcome.var = outcome.var,
                period.pre = period.pre, period.post = period.post,
                unit.tr = unit.tr, unit.co = unit.co)

####################################
### SC - point estimation with simplex
est.si  <- scest(data = df, w.constr = list(name="simplex"))
# Use print or summary methods to check results
print(est.si)
summary(est.si)

####################################
### SC - plot results
scplot(result = est.si, fig.path = figuras,
       fig.name = "germany_est_backdating", fig.format = "png", plot.range = (1960:2003),
       label.xy = list(x.lab = "Year", y.lab = "GDP per capita (thousand US dollars)"),
       event.label = list(lab = "Reunification", height = 10),
       plot.specs=list( img.width = 7,  img.height = 5, dpi=100))


#####################################################################################
#                               3. Placebos
####################################################################################

#empty dataframe to fill with placebos and RMSE 
df_rmse<-data.frame()

#iterar con cada unidad control
k<-0
for (j in unique(data$country)){
  k<-k+1
  print(paste0("placebo: ", as.character(k), " de ", as.character(length(unique(data$country))) ) )
  
  ###############  
  ### Set options for data preparation
  id.var      <- "country"                              # ID variable
  time.var    <- "year"                                 # Time variable
  period.pre  <- seq(from = 1960, to = 1990, by = 1)    # Pre-treatment period
  period.post <- (1991:2003)                            # Post-treatment period
  unit.tr     <- j                                      # Treated unit (in terms of id.var)
  unit.co     <- setdiff(unique(data$country), unit.tr) # Donors pool
  outcome.var <- "gdp"                                  # Outcome variable
  
  #al estimar el CS de alguna unidad control, excluir la unidad tratada del donors pool
  if (j!="West Germany"){
    unit.co     <- setdiff(unit.co, "West Germany")  # Donors pool
  }
  
  #################################### Data preparation #############################

  df  <-   scdata(df = data, id.var = id.var, time.var = time.var, outcome.var = outcome.var,
                  period.pre = period.pre, period.post = period.post,
                  unit.tr = unit.tr, unit.co = unit.co)
  
  ################################### point estimation ###############################
 
  est.si  <- scest(data = df, w.constr = list(name="simplex"))
  
  #########################################
  ######### crear base que contenga: 1) series observada y CS de cada unidad; 2) diferencia entre serie observada
  # y CS(para grfica); 3) RMSE de periods pre y post tratamiento (para excluir unidades de la grfica)
  
  #observado
  yo_pre<-as.data.frame(est.si$data$Y.pre)
  yo_post<-as.data.frame(est.si$data$Y.post)
  yo<-rbind.data.frame(yo_pre,yo_post)
  colnames(yo)[1]<-"observado"
  yo$t<-row.names(yo)
  row.names(yo)<-NULL
  yo$idp<-j
  yo$t<-as.numeric(seq(from=1960,to=2003))
  
  #estimado (CS)
  ye_pre<-as.data.frame(est.si$est.results$Y.pre.fit)
  ye_post<-as.data.frame(est.si$est.results$Y.post.fit)
  ye<-rbind.data.frame(ye_pre,ye_post)
  colnames(ye)[1]<-"estimado"
  ye$t<-row.names(ye)
  row.names(ye)<-NULL
  ye$idp<-j
  ye$t<-as.numeric(seq(from=1960,to=2003))
  
  #calular RMSE
  rmse<-merge.data.frame(yo,ye)
  rmse$diff<-(rmse$observado - rmse$estimado)
  rmse$diff2<-(rmse$observado - rmse$estimado)^2
  rmse$rmse_pre<-sqrt(mean(rmse$diff2[rmse$t<=1990]))
  rmse$rmse_post<-sqrt(mean(rmse$diff2[rmse$t>1990]))
  df_rmse<-rbind(df_rmse, rmse)
}

#guardar series de los placebos
write.csv(df_rmse,paste0(datos,"/rmse - placebos",".csv"), row.names = F)

##########################################################
##### calcular p-value

#catalogar unidades
df_rmse$status<-NA
df_rmse$status[df_rmse$idp=="West Germany"]<-"Unidad tratada"
df_rmse$status[df_rmse$idp!="West Germany"]<-"Placebo"

#calcular p-value
df_rmse$ratio<-df_rmse$rmse_post/df_rmse$rmse_pre
df_rmse2<-df_rmse %>% filter(t==1960) %>% select(idp,rmse_pre, rmse_post, ratio) %>% arrange(desc(ratio))
df_rmse2$rank<-seq(from = 1, to = dim(df_rmse2)[1])
n_units<- dim(df_rmse2)[1]
pval<-df_rmse2$rank[df_rmse2$idp=="West Germany"]/n_units

##########################################################
##### grfica de placebos 

p<-ggplot(data=df_rmse,aes(x=t,y=diff,group=idp))+
  geom_line(aes(color=status))+
  geom_line(data=df_rmse[df_rmse$idp=="West Germany",],color="blue")+
  geom_vline(xintercept = 1990, color="#808B96", linetype="dashed")
p

#quitar los placebos que exceden 2 veces el rmse-pre del tratado
rmse_sup<-df_rmse$rmse_pre[df_rmse$idp=="West Germany" & df_rmse$t==1960]

p<-ggplot(data=df_rmse[df_rmse$rmse_pre<=2*rmse_sup,],aes(x=t,y=diff,group=idp))+
  geom_line(aes(color=status))+
  geom_line(data=df_rmse[df_rmse$idp=="West Germany",],color="blue")+
  geom_vline(xintercept = 1990, color="#808B96", linetype="dashed")+
  ylab("GDP per capita (thousands US dollars")
p
pval

png(paste0(figuras,"/placebos.png"),
    width = 650, height = 480)
print(p)
dev.off()

#####################################################################################
#                  4. Intervalos de Prediccin
####################################################################################

####################################
### Set options for data preparation
id.var      <- "country"                              # ID variable
time.var    <- "year"                                 # Time variable
period.pre  <- seq(from = 1960, to = 1990, by = 1)    # Pre-treatment period
period.post <- (1991:2003)                            # Post-treatment period
unit.tr     <- "West Germany"                         # Treated unit (in terms of id.var)
unit.co     <- setdiff(unique(data$country), unit.tr) # Donors pool
outcome.var <- "gdp"                                  # Outcome variable

####################################
### Data preparation
df  <-   scdata(df = data, id.var = id.var, time.var = time.var, outcome.var = outcome.var,
                period.pre = period.pre, period.post = period.post,
                unit.tr = unit.tr, unit.co = unit.co)

####################################
## Set options for inference
u.alpha  <- 0.05                         # Confidence level (in-sample uncertainty)
e.alpha  <- 0.05                         # Confidence level (out-of-sample uncertainty)
sims     <- 200                          # Number of simulations
e.method <- "gaussian"                   # Estimation method for out-of-sample uncertainty
w.constr <- list(name = "simplex")       # Simplex-type constraint set

set.seed(8894)
pi.si   <- scpi(data = df, sims = sims, e.method = e.method, w.constr = w.constr,
                u.alpha = u.alpha, e.alpha = e.alpha) 

# Use print or summary methods to check results
print(pi.si)
summary(pi.si)

####################################
### SC - plot results
scplot(result = pi.si, fig.path = figuras,
       fig.name = "germany_IP", fig.format = "png", plot.range = (1960:2003),
       label.xy = list(x.lab = "Year", y.lab = "GDP per capita (thousand US dollars)"),
       x.ticks = NULL, e.out = T, event.label = list(lab = "Reunification", height = 10),
       plot.specs=list( img.width = 7,  img.height = 5, dpi=100))






